@@ -1,5 +1,5 @@
 __version__ = "0.5.0"
 
-from .indexer.runner import IndexerRunner, Info
+from .indexer import *
 from .model import (Event, EventFilter, NewBlock, NewEvents, Reorg,
                     StarkNetEvent)
